__version__ = '4'

from urbitob.co import (
    patp,
    patq,
    clan,
    sein,
    hex_to_patp,
    hex_to_patq,
    patq_to_hex,
    patp_to_hex,
    patq_to_num,
    patp_to_num,
    eq_patq,
    is_valid_patp,
    is_valid_patq,
    is_valid_pat,
)



import re
from typing import List, T, Collection

from urbitob import ob

PREFIXES = """\
dozmarbinwansamlitsighidfidlissogdirwacsabwissib\
rigsoldopmodfoglidhopdardorlorhodfolrintogsilmir\
holpaslacrovlivdalsatlibtabhanticpidtorbolfosdot\
losdilforpilramtirwintadbicdifrocwidbisdasmidlop\
rilnardapmolsanlocnovsitnidtipsicropwitnatpanmin\
ritpodmottamtolsavposnapnopsomfinfonbanmorworsip\
ronnorbotwicsocwatdolmagpicdavbidbaltimtasmallig\
sivtagpadsaldivdactansidfabtarmonranniswolmispal\
lasdismaprabtobrollatlonnodnavfignomnibpagsopral\
bilhaddocridmocpacravripfaltodtiltinhapmicfanpat\
taclabmogsimsonpinlomrictapfirhasbosbatpochactid\
havsaplindibhosdabbitbarracparloddosbortochilmac\
tomdigfilfasmithobharmighinradmashalraglagfadtop\
mophabnilnosmilfopfamdatnoldinhatnacrisfotribhoc\
nimlarfitwalrapsarnalmoslandondanladdovrivbacpol\
laptalpitnambonrostonfodponsovnocsorlavmatmipfip\
"""
SUFFIXES = """\
zodnecbudwessevpersutletfulpensytdurwepserwylsun\
rypsyxdyrnuphebpeglupdepdysputlughecryttyvsydnex\
lunmeplutseppesdelsulpedtemledtulmetwenbynhexfeb\
pyldulhetmevruttylwydtepbesdexsefwycburderneppur\
rysrebdennutsubpetrulsynregtydsupsemwynrecmegnet\
secmulnymtevwebsummutnyxrextebfushepbenmuswyxsym\
selrucdecwexsyrwetdylmynmesdetbetbeltuxtugmyrpel\
syptermebsetdutdegtexsurfeltudnuxruxrenwytnubmed\
lytdusnebrumtynseglyxpunresredfunrevrefmectedrus\
bexlebduxrynnumpyxrygryxfeptyrtustyclegnemfermer\
tenlusnussyltecmexpubrymtucfyllepdebbermughuttun\
bylsudpemdevlurdefbusbeprunmelpexdytbyttyplevmyl\
wedducfurfexnulluclennerlexrupnedlecrydlydfenwel\
nydhusrelrudneshesfetdesretdunlernyrsebhulryllud\
remlysfynwerrycsugnysnyllyndyndemluxfedsedbecmun\
lyrtesmudnytbyrsenwegfyrmurtelreptegpecnelnevfes\
"""

prefix = {i: PREFIXES[i * 3 : i * 3 + 3] for i in range(256)}
suffix = {i: SUFFIXES[i * 3 : i * 3 + 3] for i in range(256)}
iprefix = {v: k for k, v in prefix.items()}
isuffix = {v: k for k, v in suffix.items()}


def _dec_to_bin(idx: int) -> str:
    return f"{idx:08b}"


def _dec_to_hex(dec: int) -> str:
    return f"{dec:02x}"


def _bex(n: int) -> int:
    return 2**n


def _rsh(a: int, b: int, c: int) -> int:
    return c // (_bex(_bex(a) * b))


def _met(a: int, b: int, c: int = 0) -> int:
    if b == 0:
        return c
    return _met(a, _rsh(a, 1, b), c + 1)


def _end(a: int, b: int, c: int) -> int:
    return c % (_bex(_bex(a) * b))


def _patp_loop(dyy: int, tsxz: int, timp: int, trep: str) -> str:
    if timp == dyy:
        return trep
    log = _end(4, 1, tsxz)
    pre = prefix.get(_rsh(3, 1, log) // 1)
    suf = suffix.get(_end(3, 1, log) // 1)
    etc = "-" if timp % 4 != 0 else "--" if timp != 0 else ""
    res = pre + suf + etc + trep
    return _patp_loop(dyy, _rsh(4, 1, tsxz), timp + 1, res)


def _rev_chunk(s: Collection[T], size: int) -> Collection[Collection[T]]:
    rem = len(s) % size
    if rem != 0:
        yield s[:rem]
        s = s[rem:]
    for i in range(0, len(s), size):
        yield s[i : i + size]


def _pat_to_syls(name: str) -> List[str]:
    name_partition_pattern = ".{1,3}"
    remove_chars_pattern = re.compile(r"[\^~-]")
    normalized_name = remove_chars_pattern.sub("", name)
    partition_pattern = re.compile(name_partition_pattern)
    return partition_pattern.findall(normalized_name)


def patp(v: int) -> str:
    """
    Convert a number to a @p-encoded string.
    :param v:
    :return:
    """
    sxz = ob.fein(v)
    dyy = _met(4, sxz, 0)
    dyx = _met(3, sxz, 0)
    p = "~"
    if dyx <= 1:
        p += suffix.get(sxz // 1)
    else:
        p += _patp_loop(dyy, sxz, 0, "")
    return p


def patq(v: int) -> str:
    """
    Convert a number to a @q-encoded string.
    :param v:
    :return:
    """
    hex_str = _dec_to_hex(v)

    if len(hex_str) == 2:
        return "~" + suffix.get(int(hex_str, 16))

    ret = []
    for chunk in _rev_chunk(hex_str, 4):
        chunk = str(chunk).zfill(4)
        pre = chunk[:2]
        suf = chunk[2:]
        ret.append(prefix.get(int(pre, 16)) + suffix.get(int(suf, 16)))
    return "~" + "-".join(ret)


def _pat_to_hex(name: str) -> str:
    syls = _pat_to_syls(name)
    if len(syls) == 1:
        return _dec_to_hex(isuffix.get(syls[0]))
    ret = ""
    for i in range(0, len(syls), 2):
        ret += _dec_to_hex(iprefix.get(syls[i])) + _dec_to_hex(isuffix.get(syls[i + 1]))
    return ret


def patp_to_hex(name: str) -> str:
    """
    Convert a @p-encoded string to a hex-encoded string.
    :param name:
    :return:
    """
    if not is_valid_pat(name):
        raise ValueError(f"Invalid @p {name}")
    hex_value = _pat_to_hex(name)
    v = ob.fynd(int(hex_value, 16))
    hex_v = f"{v:x}"
    return f"0{hex_v}" if len(hex_v) % 2 != 0 else hex_v


def patq_to_hex(name: str) -> str:
    """
    Convert a @q-encoded string to a hex-encoded string.
    :param name:
    :return:
    """
    if not is_valid_pat(name):
        raise ValueError(f"Invalid @p {name}")
    return "00" if len(name) == 0 else _pat_to_hex(name)


def is_valid_pat(who: str) -> bool:
    """
    Weakly check if a string is a valid @p or @q value
    :param who:
    :return:
    """
    if len(who) < 4 or who[0] != "~":
        return False
    syls = _pat_to_syls(who)

    if len(syls) == 1:
        return syls[0] in isuffix
    if len(syls) % 2 != 0:
        return False

    for i in range(0, len(syls), 2):
        if syls[i] not in iprefix or syls[i + 1] not in isuffix:
            return False
    return True


def is_valid_patp(s: str) -> bool:
    """
    Validate a @p string.
    :param s:
    :return:
    """
    try:
        v = patp_to_num(s)
    except ValueError:
        return False
    return is_valid_pat(s) and s == patp(v)


def is_valid_patq(s: str) -> bool:
    """
    Validate a @q string.
    :param s:
    :return:
    """
    try:
        v = patq_to_num(s)
    except ValueError:
        return False
    p = patq(v)
    return is_valid_pat(s) and eq_patq(s, p)


def eq_patq(p: str, q: str) -> bool:
    """
    Equality comparison on @q values.
    :param p:
    :param q:
    :return:
    """
    return patq_to_hex(p).lstrip("0") == patq_to_hex(q).lstrip("0")


def patp_to_num(who: str) -> int:
    """
    Convert a @p-encoded string to a number
    :param who:
    :return:
    """
    hex_str = patp_to_hex(who)
    return int(hex_str, 16)


def patq_to_num(who: str) -> int:
    """
    Convert a @q-encoded string to a number
    :param who:
    :return:
    """
    hex_str = patq_to_hex(who)
    return int(hex_str, 16)


def hex_to_patq(hex_str: str) -> str:
    """
    Convert a hex-encoded string to a @p-encoded string
    :param hex_str: hex
    :return: @p string
    """
    return patq(int(hex_str, 16))


def hex_to_patp(hex_str: str) -> str:
    """
    Convert a hex-encoded string to a @p-encoded string
    :param hex_str: hex
    :return: @p string
    """

    return patp(int(hex_str, 16))


def clan(who: str) -> str:
    """
    Determine the ship class of a @p value.
    :param who: @p
    :return:
    """
    p = patp_to_num(who)
    wid = _met(3, p)

    if wid <= 1:
        return "galaxy"
    elif wid <= 2:
        return "star"
    elif wid <= 4:
        return "planet"
    elif wid <= 8:
        return "moon"
    else:
        return "comet"


def sein(who: str) -> str:
    """
    Determine the parent of a @p value
    :param who: @p
    :return:
    """
    p = patp_to_num(who)
    c = clan(who)

    if c == "galaxy":
        res = p
    elif c == "star":
        res = _end(3, 1, p)
    elif c == "planet":
        res = _end(4, 1, p)
    elif c == "moon":
        res = _end(5, 1, p)
    else:
        res = 0
    return patp(res)



import mmh3


def f(j: int, key: int) -> int:
    raku = [0xB76D5EED, 0xEE281300, 0x85BCAE01, 0x4B387AF7]
    lo, hi = key & 0xFF, (key & 0xFF00) // 0x100
    hash_key = f"{chr(lo)}{chr(hi)}".encode("latin_1")
    return mmh3.hash(hash_key, raku[j], signed=False)


def loop_hi_lo_init(v):
    lo = v & 0xFFFFFFFF
    hi = v & 0xFFFFFFFF00000000
    return hi, lo


def feis(m: int) -> int:
    c = fe(4, 65535, 65536, m)
    return c if c < 0xFFFFFFFF else fe(4, 65535, 65536, c)


def fe(r: int, a: int, b: int, m: int) -> int:
    right, left = divmod(m, a)
    return fe_loop(r, a, b, 1, left, right)


def fe_loop(r: int, a: int, b: int, j: int, ell: int, arr: int):
    if j > r:
        if r % 2 != 0 or arr == a:
            return (a * arr) + ell
        return (a * ell) + arr
    eff = f(j - 1, arr)
    tmp = ell + eff
    if j % 2 != 0:
        tmp = tmp % a
    else:
        tmp = tmp % b
    return fe_loop(r, a, b, j + 1, arr, tmp)


def fen_loop(a: int, b: int, j: int, ell: int, arr: int) -> int:
    if j < 1:
        return (a * arr) + ell
    eff = f(j - 1, ell)
    use_value = b if j % 2 == 0 else a
    tmp = (arr + use_value - (eff % use_value)) % use_value
    return fen_loop(a, b, j - 1, tmp, ell)


def fen(r: int, a: int, b: int, m: int) -> int:
    ale, ahh = divmod(m, a)
    if r % 2 == 1:
        ale, ahh = ahh, ale
    return fen_loop(a, b, r, ale, ahh)


def tail(v: int) -> int:
    c = fen(4, 65535, 65536, v)
    return c if c < 0xFFFFFFFF else fen(4, 65535, 65536, c)


def fein(pyn: int) -> int:
    hi, lo = loop_hi_lo_init(pyn)
    if 0x10000 <= pyn <= 0xFFFFFFFF:
        return 0x10000 + feis(pyn - 0x10000)
    if 0x100000000 <= pyn <= 0xFFFFFFFFFFFFFFFF:
        return hi | fein(lo)
    return pyn


def fynd(cry: int) -> int:
    hi, lo = loop_hi_lo_init(cry)
    if 0x10000 <= cry <= 0xFFFFFFFF:
        return 0x10000 + tail(cry - 0x10000)
    if 0x100000000 <= cry <= 0xFFFFFFFFFFFFFFFF:
        return hi | fynd(lo)
    return cry



import unittest
from urbitob.co import (
    patp,
    patq,
    clan,
    sein,
    hex_to_patp,
    hex_to_patq,
    patq_to_hex,
    patp_to_hex,
    patq_to_num,
    patp_to_num,
    eq_patq,
    is_valid_patp,
    is_valid_patq,
)


class TestPat(unittest.TestCase):
    def test_patp(self):
        self.assertEqual(patp(0), "~zod")
        self.assertEqual(patp(255), "~fes")
        self.assertEqual(patp(256), "~marzod")
        self.assertEqual(patp(65535), "~fipfes")
        self.assertEqual(patp(65536), "~dapnep-ronmyl")
        self.assertEqual(patp(14287616), "~rosmur-hobrem")
        self.assertEqual(patp(14287617), "~sallus-nodlut")
        self.assertEqual(patp(14287618), "~marder-mopdur")
        self.assertEqual(patp(14287619), "~laphec-savted")
        self.assertEqual(patp(4294967295), "~dostec-risfen")
        self.assertEqual(patp(4294967296), "~doznec-dozzod-dozzod")

    def test_patq(self):
        self.assertEqual(patq(0), "~zod")
        self.assertEqual(patq(255), "~fes")
        self.assertEqual(patq(256), "~marzod")
        self.assertEqual(patq(65535), "~fipfes")
        self.assertEqual(patq(65536), "~doznec-dozzod")
        self.assertEqual(patq(14287616), "~dozler-wanzod")
        self.assertEqual(patq(14287617), "~dozler-wannec")
        self.assertEqual(patq(14287618), "~dozler-wanbud")
        self.assertEqual(patq(14287619), "~dozler-wanwes")
        self.assertEqual(patq(4294967295), "~fipfes-fipfes")
        self.assertEqual(patq(4294967296), "~doznec-dozzod-dozzod")


class TestClan(unittest.TestCase):
    def test_clan(self):
        self.assertEqual(clan("~zod"), "galaxy")
        self.assertEqual(clan("~fes"), "galaxy")
        self.assertEqual(clan("~marzod"), "star")
        self.assertEqual(clan("~fipfes"), "star")
        self.assertEqual(clan("~dapnep-ronmyl"), "planet")
        self.assertEqual(clan("~rosmur-hobrem"), "planet")
        self.assertEqual(clan("~sallus-nodlut"), "planet")
        self.assertEqual(clan("~marder-mopdur"), "planet")
        self.assertEqual(clan("~laphec-savted"), "planet")
        self.assertEqual(clan("~dostec-risfen"), "planet")
        self.assertEqual(clan("~divrul-dalred-samhec-sidrex"), "moon")
        self.assertEqual(
            clan("~dotmec-niblyd-tocdys-ravryg--panper-hilsug-nidnev-marzod"), "comet"
        )
        self.assertRaisesRegex(ValueError, "Invalid @p", clan, "abcdefg")


class TestSein(unittest.TestCase):
    def test_sein(self):
        self.assertEqual(sein("~zod"), "~zod")
        self.assertEqual(sein("~fes"), "~fes")
        self.assertEqual(sein("~marzod"), "~zod")
        self.assertEqual(sein("~fipfes"), "~fes")
        self.assertEqual(sein("~dapnep-ronmyl"), "~zod")
        self.assertEqual(sein("~rosmur-hobrem"), "~wanzod")
        self.assertEqual(sein("~sallus-nodlut"), "~wannec")
        self.assertEqual(sein("~marder-mopdur"), "~wanbud")
        self.assertEqual(sein("~laphec-savted"), "~wanwes")
        self.assertEqual(sein("~dostec-risfen"), "~fipfes")
        self.assertEqual(sein("~divrul-dalred-samhec-sidrex"), "~samhec-sidrex")
        self.assertEqual(
            sein("~dotmec-niblyd-tocdys-ravryg--panper-hilsug-nidnev-marzod"), "~zod"
        )
        self.assertRaisesRegex(ValueError, "Invalid @p", sein, "abcdefg")


class TestPatToNum(unittest.TestCase):
    def test_patq_to_num(self):
        self.assertEqual(patq_to_num("~zod"), 0)
        self.assertEqual(patq_to_num("~fes"), 255)
        self.assertEqual(patq_to_num("~marzod"), 256)
        self.assertEqual(patq_to_num("~fipfes"), 65535)
        self.assertEqual(patq_to_num("~doznec-dozzod"), 65536)
        self.assertEqual(patq_to_num("~dozler-wanzod"), 14287616)
        self.assertEqual(patq_to_num("~dozler-wannec"), 14287617)
        self.assertEqual(patq_to_num("~dozler-wanbud"), 14287618)
        self.assertEqual(patq_to_num("~dozler-wanwes"), 14287619)
        self.assertEqual(patq_to_num("~fipfes-fipfes"), 4294967295)
        self.assertEqual(patq_to_num("~doznec-dozzod-dozzod"), 4294967296)
        self.assertRaisesRegex(ValueError, "Invalid @p", patq_to_num, "abcdefg")

    def test_patp_to_num(self):
        self.assertEqual(patp_to_num("~zod"), 0)
        self.assertEqual(patp_to_num("~fes"), 255)
        self.assertEqual(patp_to_num("~marzod"), 256)
        self.assertEqual(patp_to_num("~fipfes"), 65535)
        self.assertEqual(patp_to_num("~dapnep-ronmyl"), 65536)
        self.assertEqual(patp_to_num("~rosmur-hobrem"), 14287616)
        self.assertEqual(patp_to_num("~sallus-nodlut"), 14287617)
        self.assertEqual(patp_to_num("~marder-mopdur"), 14287618)
        self.assertEqual(patp_to_num("~laphec-savted"), 14287619)
        self.assertEqual(patp_to_num("~dostec-risfen"), 4294967295)
        self.assertEqual(patp_to_num("~doznec-dozzod-dozzod"), 4294967296)
        self.assertRaisesRegex(ValueError, "Invalid @p", patp_to_num, "abcdefg")


class TestPatToHex(unittest.TestCase):
    def test_patp_to_hex(self):
        self.assertEqual(patp_to_hex("~zod"), "00")
        self.assertEqual(patp_to_hex("~fes"), "ff")
        self.assertEqual(patp_to_hex("~marzod"), "0100")
        self.assertEqual(patp_to_hex("~fipfes"), "ffff")
        self.assertEqual(patp_to_hex("~dapnep-ronmyl"), "010000")
        self.assertEqual(patp_to_hex("~rosmur-hobrem"), "da0300")
        self.assertEqual(patp_to_hex("~sallus-nodlut"), "da0301")
        self.assertEqual(patp_to_hex("~marder-mopdur"), "da0302")
        self.assertEqual(patp_to_hex("~laphec-savted"), "da0303")
        self.assertEqual(patp_to_hex("~dostec-risfen"), "ffffffff")
        self.assertEqual(patp_to_hex("~doznec-dozzod-dozzod"), "0100000000")
        self.assertRaisesRegex(ValueError, "Invalid @p", patp_to_hex, "abcdefg")

    def test_patq_to_hex(self):
        self.assertEqual(patq_to_hex("~zod"), "00")
        self.assertEqual(patq_to_hex("~fes"), "ff")
        self.assertEqual(patq_to_hex("~marzod"), "0100")
        self.assertEqual(patq_to_hex("~fipfes"), "ffff")
        self.assertEqual(patq_to_hex("~doznec-dozzod"), "00010000")
        self.assertEqual(patq_to_hex("~dozler-wanzod"), "00da0300")
        self.assertEqual(patq_to_hex("~dozler-wannec"), "00da0301")
        self.assertEqual(patq_to_hex("~dozler-wanbud"), "00da0302")
        self.assertEqual(patq_to_hex("~dozler-wanwes"), "00da0303")
        self.assertEqual(patq_to_hex("~fipfes-fipfes"), "ffffffff")
        self.assertEqual(patq_to_hex("~doznec-dozzod-dozzod"), "000100000000")
        self.assertRaisesRegex(ValueError, "Invalid @p", patq_to_hex, "abcdefg")


class TestHexToPat(unittest.TestCase):
    def test_hex_to_patq(self):
        self.assertEqual(hex_to_patq("00"), "~zod")
        self.assertEqual(hex_to_patq("ff"), "~fes")
        self.assertEqual(hex_to_patq("0100"), "~marzod")
        self.assertEqual(hex_to_patq("ffff"), "~fipfes")
        self.assertEqual(hex_to_patq("00010000"), "~doznec-dozzod")
        self.assertEqual(hex_to_patq("00da0300"), "~dozler-wanzod")
        self.assertEqual(hex_to_patq("00da0301"), "~dozler-wannec")
        self.assertEqual(hex_to_patq("00da0302"), "~dozler-wanbud")
        self.assertEqual(hex_to_patq("00da0303"), "~dozler-wanwes")
        self.assertEqual(hex_to_patq("ffffffff"), "~fipfes-fipfes")
        self.assertEqual(hex_to_patq("000100000000"), "~doznec-dozzod-dozzod")
        self.assertEqual(
            hex_to_patq("01010101010101010102"), "~marnec-marnec-marnec-marnec-marbud"
        )
        self.assertEqual(
            hex_to_patq(
                "6d7920617765736f6d65207572626974207469636b65742c206920616d20736f206c75636b79"
            ),
            "~tastud-holruc-sidwet-salpel-taswet-holdeg-paddec-davdut-holdut-davwex-balwet-divwen-holdet-holruc"
            "-taslun-salpel-holtux-dacwex-baltud",
        )

    def test_hex_to_patp(self):
        self.assertEqual(hex_to_patp("00"), "~zod")
        self.assertEqual(hex_to_patp("ff"), "~fes")
        self.assertEqual(hex_to_patp("0100"), "~marzod")
        self.assertEqual(hex_to_patp("ffff"), "~fipfes")
        self.assertEqual(hex_to_patp("010000"), "~dapnep-ronmyl")
        self.assertEqual(hex_to_patp("da0300"), "~rosmur-hobrem")
        self.assertEqual(hex_to_patp("da0301"), "~sallus-nodlut")
        self.assertEqual(hex_to_patp("da0302"), "~marder-mopdur")
        self.assertEqual(hex_to_patp("da0303"), "~laphec-savted")
        self.assertEqual(hex_to_patp("ffffffff"), "~dostec-risfen")
        self.assertEqual(hex_to_patp("0100000000"), "~doznec-dozzod-dozzod")
        self.assertEqual(
            hex_to_patp(
                "7468697320697320736f6d6520766572792068696768207175616c69747920656e74726f7079"
            ),
            "~divmes-davset-holdet--sallun-salpel-taswet-holtex--watmeb-tarlun-picdet-magmes--holter-dacruc-timdet"
            "-divtud--holwet-maldut-padpel-sivtud",
        )


class TestEqPat(unittest.TestCase):
    def test_eq_patq(self):
        self.assertEqual(eq_patq("~dozzod-dozzod", "~zod"), True)
        self.assertEqual(eq_patq("~dozzod-mardun", "~mardun"), True)
        self.assertEqual(eq_patq("~dozzod-mardun", "~mardun-dozzod"), False)
        self.assertRaisesRegex(ValueError, "Invalid @p", eq_patq, "~zod", "abcdefg")


class TestIsValidPat(unittest.TestCase):
    def test_is_valid_patp(self):
        self.assertEqual(is_valid_patp("~zod"), True)
        self.assertEqual(is_valid_patp("~marzod"), True)
        self.assertEqual(is_valid_patp("~nidsut-tomdun"), True)
        self.assertEqual(is_valid_patp(""), False)
        self.assertEqual(is_valid_patp("~"), False)
        self.assertEqual(is_valid_patp("~hu"), False)
        self.assertEqual(is_valid_patp("~what"), False)
        self.assertEqual(is_valid_patp("sudnit-duntom"), False)

    def test_is_valid_patq(self):
        self.assertEqual(is_valid_patq("~zod"), True)
        self.assertEqual(is_valid_patq("~marzod"), True)
        self.assertEqual(is_valid_patq("~nidsut-tomdun"), True)
        self.assertEqual(is_valid_patq("~dozzod-binwes-nidsut-tomdun"), True)
        self.assertEqual(is_valid_patq(""), False)
        self.assertEqual(is_valid_patq("~"), False)
        self.assertEqual(is_valid_patq("~hu"), False)
        self.assertEqual(is_valid_patq("~what"), False)
        self.assertEqual(is_valid_patq("sudnit-duntom"), False)


# TODO is there a better way to do this
class TestReverse(unittest.TestCase):
    def test_patq_reverse(self):
        for i in range(0x9000, 0x12000, 7):
            self.assertEqual(patq_to_num(patq(i)), i)

    def test_patp_reverse(self):
        for i in range(0x9000, 0x12000, 7):
            self.assertEqual(patp_to_num(patp(i)), i)


if __name__ == "__main__":
    unittest.main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="urbitob",
    version="0.0.4",
    author="~ranren-ranlen",
    description="A python implementation of urbit-ob",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    url = "https://github.com/evening/urbit-pob",
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.6',
    install_requires=["mmh3==3.0"]
)
